
-- TODO
-- blackbox most of the targetlist table
-- finalize the initial API
-- add variables to the table for changing text color and alpha
-- properly sort tabs at registration before init_tbls are processed
-- get rid of all the +1s throughout the code

local config_name = "MDI TargetList"
local defaults = {
	fontscale             = {name = "Font Scale",            val = 1, Lval = 0, Uval = 2, dp = 2},
	target_color          = {name = "Highlight",             val = "65 255 65 65 *", Lval = 0, Uval = 255},
	list_count            = {name = "List Count",            val = 10, Lval = 1},
	name_width            = {name = "Name Width",            val = 0, Lval = 0},
	x                     = {name = "x",                     val = -1, Lval = 0, Uval = gkinterface.GetXResolution()},
	y                     = {name = "y",                     val = -1, Lval = 0, Uval = gkinterface.GetYResolution()},
	faction_filter        = {name = "Faction Filter",        val = 0},
	filter                = {name = "Filter",                val = 0},
	hide_turrets          = {name = "Hide Turrets",          val = 1},
	invert_faction_filter = {name = "Invert Faction Filter", val = 0},
	show_hive_only        = {name = "Show Hive Only",        val = 0},
	show_npc_ship_types   = {name = "Show NPC Ships",        val = 1},
	show_npc_faction      = {name = "Show NPC Faction",      val = 1},
	show_all_players      = {name = "Show All Players",      val = 1},
}

declare("targetlist", {
	version               = "4.0.0",
	config_name           = config_name,
--	ScanLock(true/false) -- use to set or test the scanlock flag to prevent TARGET_CHANGED from being processed.
})


T = {
	hide_turrets          = gkini.ReadInt   (config_name, defaults.hide_turrets.name,          defaults.hide_turrets.val) == 1,
	invert_faction_filter = gkini.ReadInt   (config_name, defaults.invert_faction_filter.name, defaults.invert_faction_filter.val) == 1,
	filter                = 0, -- legacy variable.  used to not rewrite core loops
	faction_filter        = 0, -- legacy variable.  used to not rewrite core loops
	fontscale             = gkini.ReadString(config_name, defaults.fontscale.name,             defaults.fontscale.val),
	current_target_color  = gkini.ReadString(config_name, defaults.target_color.name,          defaults.target_color.val),
	list_count            = gkini.ReadInt   (config_name, defaults.list_count.name,            defaults.list_count.val),
	name_width            = gkini.ReadInt   (config_name, defaults.name_width.name,            defaults.name_width.val),
	x                     = gkini.ReadInt   (config_name, defaults.x.name,                     defaults.x.val),
	y                     = gkini.ReadInt   (config_name, defaults.y.name,                     defaults.y.val),
	show_hive_only        = gkini.ReadInt   (config_name, defaults.show_hive_only.name,        defaults.show_hive_only.val) == 1,
	show_npc_ship_types   = gkini.ReadInt   (config_name, defaults.show_npc_ship_types.name,   defaults.show_npc_ship_types.val) == 1,
	show_npc_faction      = gkini.ReadInt   (config_name, defaults.show_npc_faction.name,      defaults.show_npc_faction.val) == 1,
	show_all_players      = gkini.ReadInt   (config_name, defaults.show_all_players.name,      defaults.show_all_players.val) == 1,
	default_target_color  = defaults.target_color.val,
	list_offset           = 1,
	playerlist            = {},
	shiplist              = {},
	id                    = "targetlist", -- used in MDI
	refresh_timer         = Timer(),
	refresh_delay         = 1000,
	current_target_listindex,
	current_target_nodeid,
	current_target_objectid,
	defaults = defaults,
	tab_identifiers = {}, -- holds init tables for tabs that are also used for identification when submitting a display update
	tab_index = 0, -- 0-based because of the use of modulo to handle tab cycling
	tabs_number_of = 0,
	tab_target_list = {}, -- holds the target list for the current tab
	tab_filters_number_of = 0, -- changed based on the registered filters for the given tab
	tab_filters = {},
}

--targetlist.T = T

if (T.x == -1) then T.x = nil end
if (T.y == -1) then T.y = nil end
if (T.name_width == 0) then T.name_width = nil end

console_print("MDI-TargetList loaded (v"..targetlist.version..").")

local function IsIUP(control) return control and pcall(iup.GetType, control) end

local function loadluafile(...)
	local f, err = loadfile((...))
	assert(f and type(f) == "function", err)
	return f(select(2, ...))
end

local function check_number(num, Lval, Uval, dp)
	-- verify a number is within the given range
	-- <num> the number to check
	-- <Lval>, <Uval>: Lower and Upper bounds of the number range; if missing, compares the number against itself
	-- [dp]:  how many decimal places to consider
	num = tonumber(num)
	Lval = tonumber(Lval) or num
	Uval = tonumber(Uval) or num
	if (num and Lval <= num and Uval >= num) then
		if (dp) then
			local decimals = tostring(num):match("%.(%d+)") or "0"
			if (decimals and decimals:len() <= dp) then
				return num -- entered number has no more decimal places than specified
			else
				return false -- too many decimal places
			end
		else
			return num -- non-decimal number is within range
		end
	end
	return false  -- you done fucked up
end

local Lock = false
function targetlist.ScanLock(state)
	if (not state) then return Lock end

	Lock = state
	if (isdeclared("targetless") and targetless) then
		targetless.api.radarlock = state
		targetless.var.lock = state
		targetless.var.scanlock = state
    end
    if (isdeclared("cargolist")) then
    end
    if (isdeclared("PR")) then
	end
end

function T:SaveSettings()
	gkini.WriteInt   (config_name, defaults.invert_faction_filter.name, self.invert_faction_filter and 1 or 0)
	--gkini.WriteInt   (config_name, defaults.filter.name,                self.filter)
	--gkini.WriteInt   (config_name, defaults.faction_filter.name,        self.faction_filter)
	gkini.WriteString(config_name, defaults.fontscale.name,             self.fontscale)
	gkini.WriteString(config_name, defaults.target_color.name,          self.current_target_color)
	gkini.WriteInt   (config_name, defaults.list_count.name,            self.list_count or defaults.list_count.val)
	gkini.WriteInt   (config_name, defaults.name_width.name,            self.name_width or defaults.name_width.val)
	gkini.WriteInt   (config_name, defaults.x.name,                     self.x or defaults.x.val)
	gkini.WriteInt   (config_name, defaults.y.name,                     self.y or defaults.y.val)
	gkini.WriteInt   (config_name, defaults.hide_turrets.name,          self.hide_turrets and 1 or 0)
	gkini.WriteInt   (config_name, defaults.show_hive_only.name,        self.show_hive_only and 1 or 0)
	gkini.WriteInt   (config_name, defaults.show_npc_ship_types.name,   self.show_npc_ship_types and 1 or 0)
	gkini.WriteInt   (config_name, defaults.show_npc_faction.name,      self.show_npc_faction and 1 or 0)
	gkini.WriteInt   (config_name, defaults.show_all_players.name,      self.show_all_players and 1 or 0)
end

local capship_type_names = {
	["Capella"] = true,
	["Goliath"] = true,
	["Heavy Assault Cruiser"] = true,
    ["Leviathan"] = true,
	["TPG Teradon Frigate"] = true,
    ["TPG Constellation Heavy Transport"] = true,
    ["Trident Light Frigate"] = true,
}

local function filter_players(factionid)
	if (T.faction_filter == 0) then
		return true
	elseif (not T.invert_faction_filter and factionid == T.faction_filter) then
		return true
	elseif (T.invert_faction_filter and factionid ~= T.faction_filter) then
		return true
	end
	return false
end

local function scan_player(charid)
	if (not charid) then return end
	local distance = GetRadarDistance(charid)
	local insert_player = false
	if (distance or T.show_all_players) then
		local name = GetPlayerName(charid)
		local ship_name = GetPrimaryShipNameOfPlayer(charid)
		local n, c = name:byte(1,2)
		local is_npc = n == 42 -- 42 = *; byte is faster than match
		local is_pc_cap = c == 32 -- 32 = space
		local factionid = GetPlayerFaction(charid)
		if     (T.hide_turrets and name:match("\*.*Turret")) then
		    --do nothing

		elseif (T.filter == 0) then -- All
			insert_player = filter_players(factionid)

		elseif (T.filter == 1
			and (is_npc and (not T.show_hive_only or T.show_hive_only and ship_name == "Robot"))
			and not is_pc_cap
		) then -- NPCs
			insert_player = filter_players(factionid)

		elseif (T.filter == 2 and not is_npc or is_pc_cap)then -- PCs
			insert_player = filter_players(factionid)

		elseif (T.filter == 3) then -- Capships
			if (filter_players(factionid)) then
				if (capship_type_names[ship_name] or (ship_name == "Robot" and name:match("Queen"))) then
					insert_player = true
				end
			end
		end

		if (is_npc and T.show_npc_ship_types and not is_pc_cap) then
			name = ship_name ~= "Robot" and "*"..(ship_name or "Unknown") or name
			name = T.show_npc_faction and name.." ("..(FactionName[factionid] or "")..")" or name
		end

		if (insert_player) then
			table.insert(T.shiplist, {
				charid = charid,
				factionid = factionid,
				name = name,
				distance = math.floor(distance or "9999")
			})
		end
	end

	return scan_player(table.remove(T.playerlist))
end

local function scan_sector()
	T.refresh_timer:Kill()
	T.playerlist = {}
	T.shiplist = {}

	ForEachPlayer(function(charid)
		if (charid ~= GetCharacterID()) then table.insert(T.playerlist, charid) end
	end)
	scan_player(table.remove(T.playerlist)) -- initiates a recursive tail call to iterate over the entire player list
	table.sort(T.shiplist, function(a, b) return a.distance < b.distance end)
	T.display_container.update_display(T.shiplist)

	T.refresh_timer:SetTimeout(T.refresh_delay, scan_sector)
end

function T.tab_filters:GetFilterPositions(tab_index)
	local filter = self[tab_index]
	local page = filter.page
	page = type(page) == "table" and page[filter.index + 1] or page
	return filter.index, page
end

function T.tab_filters:SetFilterPositions(tab_index, new_filter_index, new_page_index)
	local filter = self[tab_index]
	if (new_filter_index) then
		filter.index = new_filter_index
		return new_filter_index

	elseif (new_page_index) then
		if (type(filter.page) == "table") then
			filter.page[filter.index + 1] = new_page_index
		else
			filter.page = new_page_index
		end
		return new_page_index
	end
	error("Function called without arguments.")
end



loadluafile("api.lua", T) -- must be loaded first to allow the Targets tab to be registered so it can be created by other files
loadluafile("display.lua", T, scan_sector)
local change_target = loadluafile("commands.lua", T, scan_sector, check_number)
loadluafile("events.lua", T, IsIUP, scan_sector)
loadluafile("touch.lua", T, change_target)
loadluafile("help.lua", T)
loadluafile("options.lua", T, check_number)

-- filter pages below only serve to inform the plugin of how many filter pages there are per filter.  the titles aren't used.

T.Tab = {
	title = "Targets",
	first = true,
	columns = {{title = "", width = 0},},
	filters = {
		link_filter_positions = true,
		{
			title = "All",
			pages = {
				{title = "All"},
				{title = "Itani"},
				{title = "Serco"},
				{title = "UIT"},
			},
		},
		{
			title = "NPC",
			pages = {
				{title = "All"},
				{title = "Itani"},
				{title = "Serco"},
				{title = "UIT"},
			},
		},
		{
			title = "PC",
			pages = {
				{title = "All"},
				{title = "Itani"},
				{title = "Serco"},
				{title = "UIT"},
			},
		},
		{
			title = "Cap",
			pages = {
				{title = "All"},
				{title = "Itani"},
				{title = "Serco"},
				{title = "UIT"},
			},
		},
	},
}
targetlist.API.RegisterTab(T.Tab)
