local T, change_target = ...

-- TODO If you want to check whether the device has a touch screen you should check whether HasTouchScreen is true.

local timer = Timer()

function T:CreateRegions()
	if (not gkinterface.HasTouchScreen()) then return end

	local row = self.display[1 + T.list_offset][1]
	local lr_x = tonumber(row.x)
	local lr_y = tonumber(row.y)
	local lr_w = tonumber(row.w)
	local lr_h = tonumber(row.h)
	local lr_h = lr_h * T.list_count
	--                                                                 1    2    3    4    5    6    7    8    9    10    11    12           13           14
	self.listregion = self.listregion or gkinterface.CreateTouchRegion(nil, nil, nil, nil, nil, nil, nil, nil, nil, lr_x, lr_y, lr_x + lr_w, lr_y + lr_h, -1)

	local fcolumn = self.filters[1]
	local fr_x = tonumber(fcolumn.x)
	local fr_y = tonumber(fcolumn.y)
	local fr_w = tonumber(fcolumn.w)
	fr_w = fr_w * 4
	local fr_h = tonumber(fcolumn.h)
	--                                                                     1    2    3    4    5    6    7    8    9    10    11    12           13           14
	self.filterregion = self.filterregion or gkinterface.CreateTouchRegion(nil, nil, nil, nil, nil, nil, nil, nil, nil, fr_x, fr_y, fr_x + fr_w, fr_y + fr_h, -1)

	local tcolumn = self.tabs[1]
	local tr_x = tonumber(tcolumn.x)
	local tr_y = tonumber(tcolumn.y)
	local tr_w = tonumber(tcolumn.w)
	tr_w = tr_w * 4
	local tr_h = tonumber(tcolumn.h)
	--                                                               1    2    3    4    5    6    7    8    9    10    11    12           13           14
	self.tabregion = self.tabregion or gkinterface.CreateTouchRegion(nil, nil, nil, nil, nil, nil, nil, nil, nil, tr_x, tr_y, tr_x + tr_w, tr_y + tr_h, -1)
end

function T:DestroyRegions()
	if (not gkinterface.HasTouchScreen()) then return end

	gkinterface.DestroyTouchRegion(self.listregion)
	self.listregion = nil
	gkinterface.DestroyTouchRegion(self.filterregion)
	self.filterregion = nil
	gkinterface.DestroyTouchRegion(self.tabregion)
	self.tabregion = nil
end

-- Abort loading the rest of the file if not in TouchMode
if (not gkinterface.HasTouchScreen()) then return end

local initial_p = 0

local function list_release(event, id, percentW, percentH)
	local row_count = T.list_count
	local row_h = T.display[1][1].h
	
	if (initial_p > percentH) then
		-- Swipe Up
		if ((initial_p - percentH) < ((row_h / 4) / (row_h * row_count))) then
			-- Tap
			percentH = initial_p
		else
			gkinterface.GKProcessCommand("targetprev")
		end
	else
		-- Swipe Down
		if ((percentH - initial_p) < ((row_h / 4) / (row_h * row_count))) then
			-- Tap
			percentH = initial_p
		else
			gkinterface.GKProcessCommand("targetnext")
		end
	end
		
	if (percentH == initial_p) then
		for i = 1, row_count do
			if (percentH > ((row_h * (i - 1)) / (row_h * row_count)) and percentH < (row_h * i) / (row_h * row_count)) then
				--gkinterface.GKProcessCommand("target"..i)
				change_target(i)
				break
			end
		end
	end
	initial_p = 0
end

local function filter_release(event, id, percentW, percentH)
	local column_count = 4
	local column_w = T.filters[1].w
	if (initial_p > percentW) then
		-- Swipe Left
		if ((initial_p - percentW) < ((column_w / 4) / (column_w * column_count))) then
			-- Tap
			percentW = initial_p
		else
			gkinterface.GKProcessCommand("cyclefilter "..(T.filter - 1))
		end
	else
		-- Swipe Right
		if ((percentW - initial_p) < ((column_w / 4) / (column_w * column_count))) then
			-- Tap
			percentW = initial_p
		else
			gkinterface.GKProcessCommand("cyclefilter")
		end
	end
		
	if (percentW == initial_p) then
		for i = 1, column_count do
			if (percentW > ((column_w * (i - 1)) / (column_w * column_count)) and percentW < (column_w * i) / (column_w * column_count)) then
				if (T.filter == i - 1) then
					gkinterface.GKProcessCommand("cyclefilterpage")
				else
					gkinterface.GKProcessCommand("cyclefilter "..(i - 1))
				end
				break
			end
		end
	end
	initial_p = 0
end

local function tab_release(event, id, percentW, percentH)
local timer = Timer()
	local column_count = 4
	local column_w = T.tabs[1].w
	if (initial_p > percentW) then
		-- Swipe Left
		if ((initial_p - percentW) < ((column_w / 4) / (column_w * column_count))) then
			-- Tap
			percentW = initial_p
		else
			--gkinterface.GKProcessCommand("cycletab "..(T.tab_index - 1))
			timer:SetTimeout(1, function() gkinterface.GKProcessCommand("cycletab "..(T.tab_index - 1)) end)
		end
	else
		-- Swipe Right
		if ((percentW - initial_p) < ((column_w / 4) / (column_w * column_count))) then
			-- Tap
			percentW = initial_p
		else
			--gkinterface.GKProcessCommand("cycletab")
			timer:SetTimeout(1, function() gkinterface.GKProcessCommand("cycletab") end)
		end
	end

	if (percentW == initial_p) then
		for i = 1, column_count do
			if (i <= T.tabs_number_of and percentW > ((column_w * (i - 1)) / (column_w * column_count)) and percentW < (column_w * i) / (column_w * column_count)) then
				timer:SetTimeout(1, function() gkinterface.GKProcessCommand("cycletab "..(i - 1)) end)
				break
			end
		end
	end
	initial_p = 0
end

T.Events.TOUCH_PRESSED = function(_, event, id, percentW, percentH)
	if     (id == T.listregion) then
		initial_p = percentH
	elseif (id == T.filterregion) then
		initial_p = percentW
	elseif (id == T.tabregion) then
		initial_p = percentW
	end
end

T.Events.TOUCH_RELEASED = function(_, event, id, percentW, percentH)
	if     (id == T.listregion) then
		list_release(event, id, percentW, percentH)
	elseif (id == T.filterregion) then
		filter_release(event, id, percentW, percentH)
	elseif (id == T.tabregion) then
		tab_release(event, id, percentW, percentH)
	end
end

RegisterEvent(T.Events, "TOUCH_PRESSED")
RegisterEvent(T.Events, "TOUCH_RELEASED")
